package packageForLT05;

import java.awt.Color;
import java.awt.geom.Ellipse2D;
import java.util.List;
import java.util.Vector;

/**
 * <p>
 * The class encapsulates a data model that consists of an ordered list of three
 * dots (the first, second and third dot).
 * </p>
 * 
 * <p>
 * Each dot is understood to have a diameter (of a certain number of units,
 * where the actual physical size of the unit is left unspecified and is
 * abstracted away). Each dot is understood to have a location, which is
 * specified as an x, y coordinate in a two-dimensional plane that has its
 * origin in the upper left corner. Each dot is understood to be associated with
 * a color (the first with red, the second with blue, and the third with green).
 * </p>
 * 
 * <p>
 * This model is capable of having listeners installed upon it. Anytime this
 * model changes, the listener's <tt>changed()</tt> method will be invoked.
 * </p>
 * 
 * @author mb
 * 
 */
public class SimpleThreeDotModel {
	private Ellipse2D dot1;
	private Ellipse2D dot2;
	private Ellipse2D dot3;
	private int DIAMETER = 100;
	private int NUDGE = 4;

	private List<DotModelListener> listeners;

	/**
	 * Creates a default version of this data model in which the first dot is in
	 * location (20, 20), the second in location (30, 30), and the third in
	 * location (40, 40). All dots are given diameter of size 100.
	 */
	public SimpleThreeDotModel() {
		dot1 = new Ellipse2D.Double(20, 20, DIAMETER, DIAMETER);
		dot2 = new Ellipse2D.Double(30, 30, DIAMETER, DIAMETER);
		dot3 = new Ellipse2D.Double(40, 40, DIAMETER, DIAMETER);
		listeners = new Vector<DotModelListener>();
	}

	/**
	 * Returns the colour associated with the first dot.
	 * 
	 * @return
	 */
	public Color getColour1() {
		return Color.RED;
	}

	/**
	 * Returns the colour associated with the second dot.
	 * 
	 * @return
	 */
	public Color getColour2() {
		return Color.BLUE;
	}

	/**
	 * Returns the colour associated with the third dot.
	 * 
	 * @return
	 */
	public Color getColour3() {
		return Color.GREEN;
	}

	/**
	 * Returns the circular shape that is associated with the first dot.
	 * 
	 * @return
	 */
	public Ellipse2D getDot1() {
		return dot1;
	}

	/**
	 * Returns the circular shape that is associated with the second dot.
	 * 
	 * @return
	 */
	public Ellipse2D getDot2() {
		return dot2;
	}

	/**
	 * Returns the circular shape that is associated with the third dot.
	 * 
	 * @return
	 */
	public Ellipse2D getDot3() {
		return dot3;
	}

	/**
	 * Causes the first dot to be nudged to the right by a small amount.
	 * 
	 * If any listeners are installed on this model, they will be informed that
	 * the model has changed.
	 * 
	 */
	public void nudgeToRightDot1() {
		dot1 = new Ellipse2D.Double(dot1.getX() + NUDGE, dot1.getY() + NUDGE,
				DIAMETER, DIAMETER);
		notifyModelHasChanged();
	}

	/**
	 * Causes the second dot to have a new diameter as specified by the passed
	 * value.
	 * 
	 * If any listeners are installed on this model, they will be informed that
	 * the model has changed.
	 */
	public void changeDiameterofDot2(int newDiameter) {
		dot2 = new Ellipse2D.Double(dot2.getX(), dot2.getY(), newDiameter,
				newDiameter);
		notifyModelHasChanged();
	}

	/**
	 * Installs the passed listener on this model.
	 * 
	 * @param listener
	 */
	public void addListener(DotModelListener listener) {
		listeners.add(listener);
	}

	private void notifyModelHasChanged() {
		for (DotModelListener listener : listeners) {
			listener.changed();
		}
	}

}
